import { PluginPass, PluginObj, BabelFileMetadata, BabelFileResult } from '@babel/core';
import * as _babel_types from '@babel/types';
import { JSXElement, Expression } from '@babel/types';
import { NodePath } from '@babel/traverse';

type Macro = (this: Context, ...args: any[]) => Record<string, any> | void;
interface Options {
    macros?: Record<string, Macro>[];
    define?: Record<string, Context>[];
    apply?(path: NodePath<JSXElement>, using: Iterable<Context>): void;
    polyfill?: string | null;
}
type BabelState = PluginPass & {
    context: Context;
    opts: Options;
};

declare class Context {
    path: NodePath;
    parent: Context | undefined;
    define: Record<string, Context>;
    macros: Record<string, Macro>;
    uid: string;
    also: Set<Context>;
    props: Map<string, any>;
    usedBy: Set<NodePath<_babel_types.Node>>;
    children: Set<Context>;
    condition?: Expression | string;
    alternate?: Context;
    static get(from: NodePath): Context | undefined;
    constructor(path: NodePath, parent?: Context | BabelState, name?: string);
    get(name: string): Context[];
    macro(name: string, args: any[]): void;
}

type State = PluginPass & {
    context: Context;
    opts: Options;
};
declare namespace Plugin {
    export { Context, type Macro, type Options };
}
declare function Plugin(_compiler: any, options: Options): PluginObj<State>;

declare namespace Preset {
    interface Options extends Plugin.Options {
    }
    interface Meta extends BabelFileMetadata {
        css: string;
    }
    interface Result extends BabelFileResult {
        metadata: Meta;
        code: string;
    }
}
declare function Preset(_compiler: any, options?: Preset.Options): any;

export { Preset as default };
